package app;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

 /**
  * Allgemeiner Dialog zum Suchen und Ersetzen.<br>
  * <i>(C)opyright 2002 L.H.Hahn</i>
  * @author Dr. Lars H. Hahn
  * @version 3.1
  */


abstract public class AbstractDlgSuchenErsetzen
  extends JDialog {

  private final JTextField suchenText = new JTextField(20);
  private final JTextField ersetzenText = new JTextField(20);
  private final JButton beenden = new JButton("Beenden");
  private final JButton suchen = new JButton("Suchen");
  private final JButton ersetzen = new JButton("Ersetzen");
  private final JButton alle = new JButton("Alle Ersetzen");
  private final JFrame frame;

  /**
   * Einziger Konstruktor.
   * @param f Eltern-Fenster des Dialogs.
   */
  public AbstractDlgSuchenErsetzen(JFrame f) {
    super(f, "Suchen und Ersetzen");
    this.frame = f;
    suchenText.addInputMethodListener(new InputMethodListener() {
      public void inputMethodTextChanged(InputMethodEvent event) {
        reset();
        ersetzen.setEnabled(false);
      }
      public void caretPositionChanged(InputMethodEvent event) {
      }
    });
    beenden.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    });
    suchen.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(!suchenText.getText().equals("")) {
          if(suchen(suchenText.getText()))
            ersetzen.setEnabled(true);
          else {
            ersetzen.setEnabled(false);
            JOptionPane.showMessageDialog(frame, "Alle Register wurden durchsucht.");
          }
        }
      }
    });
    ersetzen.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(ersetzen(ersetzenText.getText()))
          ersetzen.setEnabled(false);
        else
          JOptionPane.showMessageDialog(frame, "Ersetzen nicht erfolgreich!");
      }
    });
    ersetzen.setEnabled(false);
    alle.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if(!suchenText.getText().equals(""))
          alleErsetzen();
      }
    });

    Container contentPane = this.getContentPane();
    contentPane.setLayout(new GridLayout(0,2));
    contentPane.add(new JLabel("Text zum Suchen:"));
    contentPane.add(Box.createGlue());
    contentPane.add(suchenText);
    contentPane.add(suchen);
    contentPane.add(new JLabel("Ersetzen durch:"));
    contentPane.add(Box.createGlue());
    contentPane.add(ersetzenText);
    contentPane.add(ersetzen);
    contentPane.add(beenden);
    contentPane.add(alle);

    setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);
    getRootPane().setDefaultButton(suchen);
 }

 /**
  * Zeigt den Dialog auf dem Bildschirm an, wenn er noch nicht sichtbar ist.
  */
  public void showDialog() {
    if(!isVisible()) {
      pack();
      setVisible(true);
    }
    toFront();
    reset();
  }

  /**
   * Enthlt die spezifische Suchfunktion.
   * @param text zusuchender Text
   */
  abstract protected boolean suchen(String text);

  /**
   * Enthlt die spezifische Funktion zum Ersetzen.
   * Es wird der Text (alter Text) ersetzt der zuvor per suchen() gefunden wurde.
   * @param text neuer Text, der den alten Text ersetzt
   */
  abstract protected boolean ersetzen(String text);

  /**
   * Wird aufgerufen, wenn der Benutzer den Dialog neu aufruft oder neuen Text
   * eingibt.
   */
  abstract protected void reset();

  private void alleErsetzen() {
    reset();
    int z = 0;
    while(suchen(suchenText.getText())) {
      if(ersetzen(ersetzenText.getText()))
        z++;
    }
    ersetzen.setEnabled(false);
    JOptionPane.showMessageDialog(frame, "Es wurden "+z+" Ersetzungen vorgenommen.");
  }
}