package app;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import java.util.ArrayList;

/**
 * <p>Title: jBasketTab</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Lars H. Hahn
 * @version 3.0
 */

public class CsvHandler {

  private final boolean DEBUG = false;

  private Object[][] tabelle = null;
  private char datensatzTeiler = ',';

  public CsvHandler() {
  }

  public CsvHandler(Object[][] tabelle) {
    this.tabelle = tabelle;
    if(DEBUG)
      System.out.println("CsvHandler ["+tabelle.length+"]["+tabelle[0].length+"]");
  }

  public CsvHandler(DataInputStream in) {
    try {
      read(in);
    }
    catch(IOException e) {
      if(DEBUG)
        System.out.println(e.toString());
    }
  }

  private String trimZelle(String zelle) {
    if(zelle.equals("\"\""))
      zelle = "";
    else if(zelle.startsWith("\"") && zelle.endsWith("\""))
      zelle = zelle.substring(1, zelle.length()-1);
    if(DEBUG)
      System.out.println(zelle);
    return zelle;
  }

  public String[][] getTabelle() {
    int ya = tabelle.length;
    int xa = tabelle[0].length;
    String[][] t = new String[ya][xa];
    for(int y=0; y<ya; y++)
      for(int x=0; x<xa; x++)
        t[y][x] = tabelle[y][x].toString();
    return t;
  }

  private void read(DataInputStream in)
    throws IOException, FileNotFoundException {
    ArrayList neueZeile = new ArrayList();
    ArrayList zeilen = new ArrayList();
    String zelle = "";
    int z;
    int zellenProZeile = 0;
    while((z = in.read()) != -1) {
      char c = (char)z;
      if((z == 13) || (z == 10)) {
        if(neueZeile.size() > 0) {
          neueZeile.add(trimZelle(zelle));
          zelle = "";
          zeilen.add(neueZeile);
          if(neueZeile.size() > zellenProZeile)
            zellenProZeile = neueZeile.size();
          neueZeile = new ArrayList();
        }
      }
      else if(c == datensatzTeiler) {
        if(DEBUG)
          System.out.print(zelle+", ");
        neueZeile.add(trimZelle(zelle));
        zelle = "";
      }
      else
        zelle += c;
    }
    tabelle = new Object[zeilen.size()][zellenProZeile];
    for(int y=0; y<zeilen.size(); y++) {
      tabelle[y] = ((ArrayList)zeilen.get(y)).toArray(tabelle[y]);
    }
  }


  public void write(DataOutputStream out)
    throws IOException {
    for(int y=0; y<tabelle.length; y++) {
      for(int x=0; x<tabelle[y].length; x++) {
        if(x>0)
          out.writeByte(datensatzTeiler);
        out.writeBytes("\""+tabelle[y][x]+"\"");
        if(DEBUG)
          System.out.print(" "+tabelle[y][x]);
      }
      if(DEBUG)
        System.out.println("");
      out.writeByte(13);
      out.writeByte(10);
    }
  }

  public void setDatensatzTeiler(char datensatzTeiler) {
    this.datensatzTeiler = datensatzTeiler;
  }

  public void importDlg(JFrame frame, String path) {
    CsvDateiDialog dlg = new CsvDateiDialog(frame, path, true);
    dlg.pack();
    dlg.show();
  }

  public void exportDlg(JFrame frame, String path) {
    CsvDateiDialog dlg = new CsvDateiDialog(frame, path, false);
    dlg.pack();
    dlg.show();
  }

  private class CsvDateiDialog
    extends JDialog {

    private final JPanel hauptPane = new JPanel();
    private final JPanel buttonPane = new JPanel();
    private final JTextField dateiname = new JTextField("", 20);
    private final JTextField trennzeichen = new JTextField(""+datensatzTeiler, 2);
    private final JButton buttonAuswahl = new JButton("Auswahl");
    private final JButton buttonOK = new JButton("OK");
    private final JButton buttonAbbruch = new JButton("Abbruch");
    private final JFrame frame;
    private final String path;
    private final boolean laden;

    private CsvDateiDialog(JFrame f, String p, boolean l) {
      super(f, "CSV Im-/Export", true);
      this.frame = f;
      this.laden = l;
      this.path = p;

      Container contentPane = getContentPane();
      contentPane.add(hauptPane, BorderLayout.CENTER);
      contentPane.add(buttonPane, BorderLayout.SOUTH);
      hauptPane.setLayout(new GridLayout(0,2));
      hauptPane.add(new JLabel("Dateiname:"));
      hauptPane.add(dateiname);
      hauptPane.add(Box.createGlue());
      hauptPane.add(buttonAuswahl);
      hauptPane.add(new JLabel("Trennzeichen:"));
      hauptPane.add(trennzeichen);
      buttonPane.add(buttonOK);
      buttonPane.add(buttonAbbruch);

      buttonAuswahl.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JFileChooser dateidialog  = new JFileChooser(path);
          if(laden) {
            if(dateidialog.showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
              dateiname.setText(dateidialog.getSelectedFile().getAbsolutePath());
            }
          }
          else {
            if(dateidialog.showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
              String text = dateidialog.getSelectedFile().getAbsolutePath();
              if(!text.toLowerCase().endsWith(".csv"))
                text += ".csv";
              dateiname.setText(text);
            }
          }
        }
      });
      buttonOK.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          datensatzTeiler = trennzeichen.getText().toCharArray()[0];
          try {
            if(laden) {
              FileInputStream fis = new FileInputStream(dateiname.getText());
              DataInputStream in = new DataInputStream(fis);
              read(in);
              fis.close();
            }
            else {
              FileOutputStream fos = new FileOutputStream(dateiname.getText());
              DataOutputStream out = new DataOutputStream(fos);
              write(out);
              out.flush();
              fos.close();
            }
          }
          catch(IOException ex) {
            if(DEBUG)
              System.out.println(ex.toString());
          }
          setVisible(false);
        }
      });
      buttonAbbruch.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          setVisible(false);
        }
      });
      getRootPane().setDefaultButton(buttonAbbruch);
    }

  }

  public static void main(String[] args)
   throws Exception {
    FileInputStream fis = new FileInputStream("test-out.csv");
    DataInputStream in = new DataInputStream(fis);
    CsvHandler handler = new CsvHandler(in);
    fis.close();

    FileOutputStream fos = new FileOutputStream("test-out2.csv");
    DataOutputStream out = new DataOutputStream(fos);
    handler.write(out);
    out.flush();
    fos.close();
  }
}
