package app;

import java.awt.*;

/**
 * Erzeugt eine Tabelle, die ausdruckbar und am Bildschirm anzeigbar ist.<br>
 * <i>(C)opyright 2002 L.H.Hahn</i>
 * @author Dr. Lars H. Hahn
 * @version 3.0
 */

public class TabellenPane
  extends AbstractPrintablePane {
  private final static boolean DEBUG = false;

  private final Object[][] tabelle;

  /**
   * @param tabelle Tabelle, die angezeigt werden soll. Zur Zeit werden nur
   *                Strings ausgewertet.
   */
  public TabellenPane(String ueberschrift, Object[][] tabelle) {
    super(ueberschrift);
    this.tabelle = tabelle;
    setFont("SansSerif", 12);
  }

  public void setFont(String name, int punkt) {
    super.setFont(name, punkt);
    FontMetrics fontMetrics = getFontMetrics(getFont());
    int hoehe = (fontMetrics.getHeight() + 2) * tabelle.length + 15 + fontMetrics.getHeight();
    setPreferredSize(new Dimension(500, hoehe));
    if(DEBUG)
      System.out.println("font="+name+","+punkt+" fenster=[500,"+hoehe+"]");
  }

  public void zeichnen(Graphics g, int x, int y, int dx, int dy) {
    FontMetrics fontMetrics = g.getFontMetrics();
    int textHoehe       = fontMetrics.getHeight();
    int zellenProZeile  = tabelle[0].length;
    int zellenProSpalte = tabelle.length;
    int zellenBreite    = dx / zellenProZeile;
    int maxZellenHoehe  = dy / zellenProSpalte;
    int zellenHoehe     = (maxZellenHoehe > textHoehe+2)?textHoehe+2:maxZellenHoehe;
    g.setColor(Color.lightGray);
    for(int zy=1; zy<zellenProSpalte; zy+=2)
      g.fillRect(x, y+zellenHoehe*zy, dx, zellenHoehe);
    g.setColor(Color.black);
    for(int zy=0; zy<=zellenProSpalte; zy++)
      g.drawLine(x, y+zellenHoehe*zy, x+dx, y+zellenHoehe*zy);
    for(int zx=0; zx<zellenProZeile; zx++) {
      g.drawLine(x+zellenBreite*zx, y, x+zellenBreite*zx, y+zellenHoehe*zellenProSpalte);
      for(int zy=0; zy<zellenProSpalte; zy++) {
        String text = tabelle[zy][zx].toString();
        while(fontMetrics.stringWidth(text) > zellenBreite-3)
          text = text.substring(0, text.length()-1);
        g.drawString(text, x+zellenBreite*zx+2 ,y+zellenHoehe*zy+textHoehe-1);
      }
    }
    g.drawLine(x+dx, y, x+dx, y+zellenHoehe*zellenProSpalte);
  }

}