package haushalt;
import haushalt.werte.*;
import app.Applikation;

import java.io.*;
import javax.swing.table.*;
import java.util.ArrayList;


/**
 * @author Lars H. Hahn
 * @version 1.1
 */

public class AutoBuchung
  extends AbstractTableModel
  implements Speicherbar {

  private final static boolean DEBUG = false;

  public final String[] intervallNamen =
    {"Monat","Quartal","Halbjahr","Jahr"};

  private final String[] spaltenNamen =
    {"Datum","Buchungstext","Kategorie","Betrag ()","Register","Intervall"};

  private ArrayList buchungen = new ArrayList();
  private ArrayList registers = new ArrayList();
  private ArrayList intervalle = new ArrayList();

  public void delBuchung(int index) {
    if(index < buchungen.size()) {
      buchungen.remove(index);
      registers.remove(index);
      intervalle.remove(index);
      this.fireTableRowsDeleted(index, index);
    }
  }

  public void allesLoeschen() {
    buchungen.clear();
    registers.clear();
    intervalle.clear();
  }

  public int ausfuehren() {
    int zaehler = 0;
    StandardBuchung buchung;
    Register register;
    String intervall;
    for(int i=0; i<buchungen.size(); i++) {
      buchung = (StandardBuchung)buchungen.get(i);
      register = (Register)registers.get(i);
      while(buchung.getDatum().compareTo(new Datum()) <= 0) {
        register.einfuegenBuchung(buchung);
        if(DEBUG)
          System.out.println("AutoBuchung ausgefhrt: "+buchung.getBuchungsZeile(register));
        intervall = (String)intervalle.get(i);
        buchung = (StandardBuchung)buchung.clone(); // sonst wrde die abgelegte Buchung gendert
        buchung.setDatum(getFolgeDatum(buchung.getDatum(), intervall));
        buchungen.set(i, buchung);
        zaehler++;
      }
    }
    return zaehler;
  }

  private Datum getFolgeDatum(Datum datum, String intervall) {
    int monat = datum.getMonat();
    int jahr = datum.getJahr();
    if(intervall.equals(intervallNamen[0])) // Monat
      monat += 1;
    if(intervall.equals(intervallNamen[1])) // Quartal
      monat += 3;
    if(intervall.equals(intervallNamen[2])) // Halbjahr
      monat += 6;
    if(intervall.equals(intervallNamen[3])) // Jahr
      jahr += 1;
    if (monat > 12) {
      monat -= 12;
      jahr ++;
    }
    return new Datum(datum.getTag(), monat, jahr);
  }

  public int getColumnCount() {
    return spaltenNamen.length;
  }

  public Object getValueAt(int rowIndex, int columnIndex) {
    if((rowIndex == buchungen.size()) && (columnIndex == 4))
      return Register.getRegister()[0];
    if(columnIndex == 4)
      return registers.get(rowIndex);
    if((rowIndex == buchungen.size()) && (columnIndex == 5))
      return intervallNamen[0];
    if(columnIndex == 5)
      return intervalle.get(rowIndex);
    if(rowIndex == buchungen.size())
       return new StandardBuchung();
    return buchungen.get(rowIndex);
  }

  public int getRowCount() {
    return buchungen.size()+1;
  }

  public String getColumnName(int columnIndex) {
    return spaltenNamen[columnIndex];
  }

  public Class getColumnClass(int columnIndex) {
    if(columnIndex == 4)
      return Register.class;
    if(columnIndex == 5)
      return String.class;
    return Buchung.class;
  }

  public boolean isCellEditable(int rowIndex, int columnIndex) {
    return true;
  }

  public void setValueAt(Object value, int rowIndex, int columnIndex) {
    if(rowIndex == buchungen.size()) { // neue Buchung
      if (columnIndex == 4) {
        buchungen.add(new StandardBuchung());
        registers.add((Register)value);
        intervalle.add(intervallNamen[0]);
      }
      else if (columnIndex == 5) {
        buchungen.add(new StandardBuchung());
        registers.add(Register.getRegister()[0]);
        intervalle.add((String)value);
      }
      else {
        buchungen.add((StandardBuchung)value);
        registers.add(Register.getRegister()[0]);
        intervalle.add(intervallNamen[0]);
      }
      this.fireTableRowsInserted(rowIndex, rowIndex);
    }
    else {
      if (columnIndex == 4)
        registers.set(rowIndex, (Register)value);
      else if (columnIndex == 5)
        intervalle.set(rowIndex, (String)value);
      else
        buchungen.set(rowIndex, (StandardBuchung)value);
      this.fireTableRowsUpdated(rowIndex, rowIndex);
    }
    Applikation.geaendert(true);
  }

  /*
   * Methoden des Interface 'Speicherbar'
   */

  public void laden(DataInputStream in)
    throws IOException {
    int size = in.readInt();
    buchungen.ensureCapacity(size);
    for(int i=0; i<size; i++) {
      // Buchung laden:
      StandardBuchung buchung = null;
      String typ = in.readUTF();
      if(typ.equals("StandardBuchung"))
        buchung = new StandardBuchung();
      else
        Applikation.fehlerAusgabe.println("-E- FATAL! AutoBuchung: Falscher Buchungstyp: "+typ);
      buchung.laden(in);
      // Register laden:
      String reg = in.readUTF();
      Register register = Register.findeRegister(reg);
      // Problem: Das Quell-Register der Umbuchung gibt es vielleicht noch nicht:
      if(register == null) { // also erzeugen!
        register = Register.erzeugeRegister(reg);
        Applikation.fehlerAusgabe.println("-W- Automatische Buchungen: Das unbekannte Register '"+reg+"' wurde erzeugt.");
      }
      // Intervall laden:
      String intervall = in.readUTF();

      buchungen.add(buchung);
      registers.add(register);
      intervalle.add(intervall);
    }
  }

  public void speichern(DataOutputStream out)
    throws IOException {
    out.writeInt(buchungen.size());
    for(int i=0; i<buchungen.size(); i++) {
      StandardBuchung buchung = (StandardBuchung)buchungen.get(i);
      buchung.speichern(out);
      Register register = (Register)registers.get(i);
      out.writeUTF(register.toString());
      String intervall = (String)intervalle.get(i);
      out.writeUTF(intervall);
    }
  }
}