package haushalt;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 * @author Lars H. Hahn
 * @version 1.1
 */

public class BuchungstextField
  extends JTextField
  implements BuchungsField {

  final private Buchung buchung;
  final private BuchungstextField me = this;
  final private MyGlassPane myGlassPane = new MyGlassPane();

  public BuchungstextField(Buchung buchung, JFrame masterFrame) {
    this.buchung = buchung;
    masterFrame.setGlassPane(myGlassPane);
    setText(buchung.getText());
  }

  public Buchung aktualisiereBuchung() {
    StandardBuchung gemerkteBuchung = Register.findeGemerkteBuchung(getText());
    if((gemerkteBuchung == null) || (buchung.getClass() == Umbuchung.class))
      buchung.setText(getText());
    else if(!buchung.getText().equals(getText())) // nicht den gleichen Buchnungstext mit alten Werten ueberschreiben!
      ((StandardBuchung)buchung).werteUebernehmen(gemerkteBuchung);
    return buchung;
  }

  protected Document createDefaultModel() {
    return new BuchungstextDocument();
  }

  private class BuchungstextDocument
    extends PlainDocument {
    public void insertString(int offs, String str, AttributeSet a)
      throws BadLocationException {
      super.insertString(offs, str, a);
      StandardBuchung gemerkteBuchung = Register.findeGemerkteBuchung(getText(0, getLength()));
      if((gemerkteBuchung != null) && (buchung.getClass() == StandardBuchung.class)){
        SplitBetrag splitBetrag = gemerkteBuchung.getSplitBetrag();
        String text = gemerkteBuchung.getText()+" / "+splitBetrag+" / "+splitBetrag.getWert();
        Point xy = SwingUtilities.convertPoint(me.getParent(), me.getX(), me.getY(), myGlassPane);
        myGlassPane.set(text, (int)xy.getX(), (int)xy.getY());
        myGlassPane.setVisible(true);
      }
    }

    public String getText(int offset, int length)
      throws BadLocationException {
      // wenn der Inhalt abgefragt wurde, kann die gemerkte Buchung ausgeblendet werden
      myGlassPane.setVisible(false);
      return super.getText(offset, length);
    }
  }

  private class MyGlassPane
    extends JComponent {

    private String text = "";
    private int x = 0;
    private int y = 0;

    public void paint(Graphics g) {
    if(text != "") {
      FontMetrics fontMetrics = g.getFontMetrics();
      int dx = fontMetrics.stringWidth(text) + 1;
      int dy = fontMetrics.getAscent() + fontMetrics.getDescent();
      g.setColor(Color.red);
      g.fillRect(x,y-fontMetrics.getHeight(),dx,dy);
      g.setColor(Color.black);
      g.drawString(text, x+1, y+fontMetrics.getAscent()-fontMetrics.getHeight());
    }
    }

    public void set(String text, int x, int y) {
      this.text = text;
      this.x = x;
      this.y = y;
      repaint();
    }

  }

}