package haushalt;
import haushalt.werte.Euro;

import java.util.ArrayList;
import java.util.Collections;

/**
 * @author Lars H. Hahn
 * @version 1.2
 */

public class Kategorie
  implements Comparable {

  public static final Kategorie SONSTIGES = new Kategorie("Sonstiges");
  private String name;
  private Euro summe;
  private static ArrayList kategorieListe = new ArrayList();
  private final static boolean dummy = kategorieListe.add(SONSTIGES);

  private Kategorie(String name) {
    this.name = name;
    summe = new Euro();
  }

  public String toString() {
    return name;
  }

  public String getHauptkategorie() {
    int n = name.indexOf(":");
    if(n == -1)
      return name; // weil selber Hauptkategorie
    return name.substring(0, n);
  }

  public void addEuro(Euro betrag) {
    summe = summe.add(betrag);
  }

  /*
   * STATIC --------------------------------------------------------------------
   */

  static public boolean isKategorie(String str) {
    for(int i=0; i<kategorieListe.size(); i++) {
      if(str.equals(kategorieListe.get(i).toString()))
        return true;
    }
    return false;
  }

  static public Kategorie lookup(String str) {
    Kategorie kategorie = new Kategorie(str);
    int pos = Collections.binarySearch(kategorieListe, kategorie);
    if (pos < 0) {
      kategorieListe.add(-pos-1, kategorie);
      return kategorie;
    }
    return (Kategorie)kategorieListe.get(pos);
  }

  static public void allesLoeschen() {
    kategorieListe.clear();
    kategorieListe.add(SONSTIGES);
  }

  static public void summenLoeschen() {
    int size = kategorieListe.size();
    for(int i=0;i<size;i++)
      ((Kategorie)kategorieListe.get(i)).summe = new Euro();
  }

  static private boolean isHauptkategorie(int idx) {
    return kategorieListe.get(idx).toString().indexOf(":") == -1;
  }

  static public Euro[] getSummen(boolean unterkategorien) {
    String[] kategorien = getKategorien(unterkategorien);
    int size = kategorieListe.size();
    Euro[] summen = new Euro[kategorien.length];

    if(unterkategorien) {
      for(int i=0; i<size; i++) {
        Kategorie kat = (Kategorie)kategorieListe.get(i);
        summen[i] = kat.summe;
      }
    }
    else {
      int c = 0;
      summen[c] = new Euro(); // der Erste in der Liste muss initialisiert werden!
      for(int i=0; i<size; i++) {
        Kategorie kat = (Kategorie)kategorieListe.get(i);
        if(kat.getHauptkategorie().equals(kategorien[c]))
          summen[c] = summen[c].add(kat.summe);
        else
          summen[++c] = kat.summe;
      }
    }

    return summen;
  }

  static public String[] getKategorien(boolean unterkategorien) {
    int size = kategorieListe.size();
    String[] kategorien = new String[size];
    int katZaehler = 0;
    for(int i=0; i<size; i++) {
      Kategorie kategorie = (Kategorie)kategorieListe.get(i);
      if(unterkategorien)
        kategorien[i] = kategorie.name;
      else {
        String name = kategorie.getHauptkategorie();
        if((katZaehler == 0) || !name.equals(kategorien[katZaehler-1]))
          kategorien[katZaehler++] = name;
      }
    }
    if(unterkategorien)
      return kategorien;
    String[] haupt = new String[katZaehler];
    System.arraycopy(kategorien,0,haupt,0,katZaehler);
    return haupt;
  }

  /*
   * Methoden fuer Interface: Comparable
   */

  public int compareTo(Object obj) {
    return name.compareToIgnoreCase(((Kategorie)obj).name);
  }

 }