package haushalt;
import haushalt.werte.*;
import app.*;

import java.io.*;
import javax.swing.table.*;
import javax.swing.JTable;
import javax.swing.JOptionPane;
import java.util.ArrayList;
import java.util.Collections;
import java.awt.event.*;

/**
 * @author Lars H. Hahn
 * @version 1.2
 */

public class Register
  extends AbstractTableModel
  implements Speicherbar, KeyListener {

  private final static boolean DEBUG = false;

  private final String[] spaltenNamen =
    {"Datum","Buchungstext","Kategorie","Betrag","Saldo"};

  private String name;
  private ArrayList buchungen = new ArrayList();

  private static ArrayList registerListe = new ArrayList();
  private static ArrayList gemerkteBuchungen = new ArrayList();
  private static ArrayList gemerkteBuchungenText = new ArrayList();
  private static boolean gemerkteBuchungenVerwenden = true;

  private Register(String name) {
    this.name = name;
  }

  public String toString() {
    return name;
  }

  public String[][] csvExport() {
    int anzahl = buchungen.size();
    String[][] text = new String[anzahl][3];
    for(int i=0; i<anzahl; i++) {
      Buchung buchung = (Buchung)buchungen.get(i);
      text[i][0] = buchung.getDatum().toString();
      text[i][1] = buchung.getText();
      text[i][2] = buchung.getWert().toString();
    }
    return text;
  }

  public String[] getBuchungen(ZeitIntervall intervall) {
    ArrayList text = new ArrayList();
    for(int i=0; i<buchungen.size(); i++) {
      Buchung buchung = (Buchung)buchungen.get(i);
      if(buchung.getDatum().istImIntervall(intervall))
        text.add(buchung.getBuchungsZeile(this));
    }
    return (String[])text.toArray(new String[text.size()]);
  }

  public void importQuicken(InputStream in, boolean dm)
  throws IOException {
    boolean loop = true;
    Applikation.fehlerAusgabe.println("-I- QuickenImport: "+Buchung.leseZeile(in));
    while(loop) {
      Buchung buchung;
      try {
        if((buchung = Buchung.importQuicken(in, dm)) == null)
          loop = false;
        else {
          if((buchung.getClass() == StandardBuchung.class))
            einfuegenBuchung((StandardBuchung)buchung);
          else if (buchung.getWert().compareTo(Euro.NULL_EURO) > 0)
            einfuegenBuchung((Umbuchung)buchung); // neg. Zwillingsbuchung rausfiltern
                                                  // NICHT die pos. wg. Eroefnungssaldo!
        }
      }
      catch(Buchung.ImportQuickenException e) { // Umbuchung in SplitBuchung gefunden!
        while((buchung = e.getBuchung()) != null) {
          if((buchung.getClass() == StandardBuchung.class))
            einfuegenBuchung((StandardBuchung)buchung);
          else if (buchung.getWert().compareTo(Euro.NULL_EURO) > 0)
            einfuegenBuchung((Umbuchung)buchung); // neg. Zwillingsbuchung rausfiltern
        }
      }
    }
  }

  public void einfuegenBuchung(Umbuchung umbuchung) {
    // Falls eine Umbuchung eingefuegt wird muss eine Zwillingsbuchung
    // erzeugt werden!
    if(umbuchung.getQuellRegister() == this) {
      // wiedereingefuegte Umbuchungen koennen hier landen:
      einsortierenBuchung(umbuchung);
      if((umbuchung.getZielRegister() == null) ||
         (umbuchung.getZielRegister() == this))
        // Selbst-Umbuchung fuer Eroeffnungssaldo; nicht doppelt buchen
        umbuchung.setZielRegister(this);
      else
        umbuchung.getZielRegister().einsortierenBuchung(umbuchung);
    }
    else {
      // hier geht es in der Regel los:
      umbuchung.setZielRegister(this); // kann noch 'null' sein!
      // hier einsortieren
      einsortierenBuchung(umbuchung);
      // jetzt im Quellregister einsortieren ...
      umbuchung.getQuellRegister().einsortierenBuchung(umbuchung);
    }
  }

  public void einfuegenBuchung(StandardBuchung buchung) {
    einsortierenBuchung(buchung);
    buchungMerken(buchung);
  }

  public void buchungMerken(StandardBuchung buchung) {
    // normale Buchungen merken (keine Umbuchungen!)
    int pos = Collections.binarySearch(gemerkteBuchungenText, buchung.getText());
    if (pos < 0) { // so eine Buchung gibt es noch nicht: Einfuegen
      gemerkteBuchungen.add(-pos-1, buchung);
      gemerkteBuchungenText.add(-pos-1, buchung.getText());
    }
    else {         // Buchung gibt es schon: Ueberschreiben
      gemerkteBuchungen.set(pos, buchung);
      gemerkteBuchungenText.set(pos, buchung.getText());
    }
  }

  public void entferneBuchung(Buchung buchung) {
    int idx = buchungen.indexOf(buchung);
    buchungen.remove(idx);
    this.fireTableRowsDeleted(idx,idx);
    Haushalt.geaendert(true); // damit beim Beenden ggf. auf das fehlende Speichern hingewiesen wird
  }

  public void killBuchung(Buchung buchung) {
    if((buchung.getClass() == Umbuchung.class) &&
       (((Umbuchung)buchung).getQuellRegister() != this))
         ((Umbuchung)buchung).getQuellRegister().entferneBuchung(buchung);
    if((buchung.getClass() == Umbuchung.class) &&
       (((Umbuchung)buchung).getZielRegister() != this))
         ((Umbuchung)buchung).getZielRegister().entferneBuchung(buchung);
    entferneBuchung(buchung);
  }

  public void einsortierenBuchung(Buchung buchung) {
    int size = buchungen.size();
    int pos = -1;
    for(int i=0; i<size; i++)
      if(buchung.compareTo(buchungen.get(i)) >= 0)
        pos = i;
    if(pos == size-1) {  // neue Buchung ans Ende
      buchungen.add(buchung);
      this.fireTableRowsInserted(pos, pos);
    }
    else {              // neue Buchung einfuegen
      buchungen.add(pos+1, buchung);
      this.fireTableDataChanged(); // Salden mssen angepasst werden
    }
    Haushalt.geaendert(true); // damit beim Beenden ggf. auf das fehlende Speichern hingewiesen wird
  }

  public void registerVereinigen(String konto) {
    Register register = findeRegister(konto);
    while(register.buchungen.size() > 0) {
      if(register.buchungen.get(0).getClass() == StandardBuchung.class) {
        StandardBuchung standardBuchung = (StandardBuchung)register.buchungen.get(0);
        einfuegenBuchung(standardBuchung);
        register.entferneBuchung(standardBuchung);
      }
      else {
        Umbuchung umbuchung = (Umbuchung)register.buchungen.get(0);
        register.killBuchung(umbuchung);
        if((umbuchung.getQuellRegister() == register) &&
           (umbuchung.getZielRegister() == register)) {
          umbuchung.setQuellRegister(this);
          umbuchung.setZielRegister(this);
          einfuegenBuchung(umbuchung);
        }
        else if((umbuchung.getQuellRegister() == register) &&
                (umbuchung.getZielRegister() != this)) {
          umbuchung.setQuellRegister(this);
          einfuegenBuchung(umbuchung);
        }
        else if((umbuchung.getQuellRegister() != this) &&
                (umbuchung.getZielRegister() == register)) {
          umbuchung.setZielRegister(this);
          einfuegenBuchung(umbuchung);
        }
      }
    }
    registerListe.remove(register);
  }

  public Euro getSaldo(Datum datum) {
    Euro saldo = new Euro();
    int i = 0;
    while((i < buchungen.size()) &&
      (datum.compareTo(((Buchung)buchungen.get(i)).getDatum()) > 0)) {
      if((buchungen.get(i).getClass() == StandardBuchung.class) ||
         (((Umbuchung)buchungen.get(i)).getZielRegister() == this))
        saldo = saldo.add(((Buchung)buchungen.get(i)).getWert());
      else
        // Falls dieses Register Quelle einer Umbuchung ist: Subtrahieren!
        saldo = saldo.sub(((Buchung)buchungen.get(i)).getWert());
      i++;
    }
    return saldo;
  }

  public void summieren(ZeitIntervall intervall, boolean unterkategorien) {
    for(int j=0;j<buchungen.size();j++)
      ((Buchung)buchungen.get(j)).summieren(intervall);
  }

  // private ------------------------------------------------------------------
  private Euro getSaldo(int row) {
    Euro saldo = new Euro();
    for(int i=0; i<=row; i++)
      if((buchungen.get(i).getClass() == StandardBuchung.class) ||
         (((Umbuchung)buchungen.get(i)).getZielRegister() == this))
        saldo = saldo.add(((Buchung)buchungen.get(i)).getWert());
      else
        // Falls dieses Register Quelle einer Umbuchung ist: Subtrahieren!
        saldo = saldo.sub(((Buchung)buchungen.get(i)).getWert());
    return saldo;
  }

  // static -------------------------------------------------------------------
  private static int registerSuchIdx = 0;
  private static int buchungSuchIdx = 0;
  private static String gefundenerText = null;

  public static void resetSuchIdx() {
    registerSuchIdx = 0;
    buchungSuchIdx = 0;
  }

  public static boolean suchen(OberflaecheHaushalt of, String text) {
    for(int i=registerSuchIdx; i<registerListe.size(); i++) {
      Register register = (Register)registerListe.get(i);
      for(int j=buchungSuchIdx; j<register.buchungen.size(); j++) {
        buchungSuchIdx++;
        Buchung buchung = (Buchung)register.buchungen.get(j);
        if(buchung.sucheText(text)) {
          of.selektiereBuchung(register, j);
          gefundenerText = text;
          return true;
        }
      }
      buchungSuchIdx = 0;
      registerSuchIdx++;
    }
    registerSuchIdx = 0;
    gefundenerText = null;
    return false;
  }

  public static boolean ersetzen(String neuerText) {
    Register register = (Register)registerListe.get(registerSuchIdx);
    Buchung buchung = (Buchung)register.buchungen.get(buchungSuchIdx-1);
    if(buchung.ersetzeText(gefundenerText, neuerText)) {
      register.fireTableRowsUpdated(buchungSuchIdx-1, buchungSuchIdx-1);
      Haushalt.geaendert(true); // damit beim Beenden ggf. auf das fehlende Speichern hingewiesen wird
      return true;
    }
    return false;
  }

  public static String[] getBuchungen(ZeitIntervall intervall, String kat, boolean unterkategorien) {
    Kategorie kategorie = Kategorie.lookup(kat);
    ArrayList text = new ArrayList();
    Euro summe = new Euro();
    for(int i=0; i<registerListe.size(); i++) {
      Register register = (Register)registerListe.get(i);
      for(int j=0; j<register.buchungen.size(); j++) {
        Buchung buchung = (Buchung)register.buchungen.get(j);
        if(buchung.getDatum().istImIntervall(intervall)) {
          String zeile = buchung.getBuchungsZeile(register, kategorie, unterkategorien);
          if(zeile != null) {
            summe = summe.add(buchung.getWert(kategorie, unterkategorien));
            int size = text.size();
            int pos = -1;
            for(int k=0; k<size; k++)
              if(buchung.getDatum().compareTo(new Datum(text.get(k).toString().substring(0,8))) >= 0)
                pos = k;
            if(pos == size-1)  // neue Zeilr ans Ende
              text.add(zeile);
            else              // neue Zeile einfuegen
              text.add(pos+1, zeile);
          }
        }
      }
    }
    text.add(" \tS U M M E:\t \t"+summe);
    return (String[])text.toArray(new String[text.size()]);
  }

  public static Euro getGesamtSaldo(Datum datum) {
    Euro saldo = new Euro();
    for(int i=0; i<registerListe.size(); i++) {
      Register register = (Register)registerListe.get(i);
      saldo = saldo.add(register.getSaldo(datum));
    }
    return saldo;
  }

  public static void allesLoeschen() {
    registerListe.clear();
    gemerkteBuchungen.clear();
    gemerkteBuchungenText.clear();
  }

  public static void entfernenAlteBuchungen(Datum datum) {
    int anzahlRegister = registerListe.size();
    int[] pos = new int[anzahlRegister];
    Euro[] salden = new Euro[anzahlRegister];
    // erstmal die Salden aller Register ermitteln ...
    for(int i=0; i<anzahlRegister; i++) {
      Register register = (Register)registerListe.get(i);
      pos[i] = -1;
      for(int j=0; j<register.buchungen.size();j++)
        if(((Buchung)register.buchungen.get(j)).compareTo(datum) < 0)
          pos[i] = j;
      if(pos[i] > -1)
        salden[i] = register.getSaldo(pos[i]);
    }
    // ... und dann ggf. alte Buchungen loeschen.
    for(int i=0; i<anzahlRegister; i++) {
      Register register = (Register)registerListe.get(i);
      if(pos[i] > -1) {
        for(int j=0;j<=pos[i];j++)
          register.buchungen.remove(0);
        register.fireTableRowsDeleted(0, pos[i]);
        register.einfuegenBuchung(new Umbuchung(datum,"Erffnungssaldo",register.name,salden[i]));
        Applikation.fehlerAusgabe.println("-I- Im Register "+register.name+" wurden "+pos[i]+" Buchungen gelscht!");
      }
    }
  }

  public static void setGemerkteBuchungenVerwenden(boolean b) {
    gemerkteBuchungenVerwenden = b;
  }

  public static StandardBuchung findeGemerkteBuchung(String prefix) {
    if(gemerkteBuchungenVerwenden)
      for(int i=0; i<gemerkteBuchungenText.size(); i++)
        if(((String)gemerkteBuchungenText.get(i)).startsWith(prefix))
          return (StandardBuchung)gemerkteBuchungen.get(i);
    return null;
  }

  public static Register findeRegister(String konto) {
    for(int i=0; i<registerListe.size(); i++) {
      if(konto.equals(registerListe.get(i).toString()))
        return (Register)registerListe.get(i);
    }
    return null;
  }

  public static Register erzeugeRegister(String inName) {
    String genName = inName;
    boolean nameVorhanden;
    int count = 0;
    do {
      nameVorhanden = false;
      for(int i=0; i<registerListe.size(); i++)
        if(genName.equalsIgnoreCase(registerListe.get(i).toString()))
          nameVorhanden = true;
      if(nameVorhanden)
        genName = inName + " (" + ++count + ")";
    } while(nameVorhanden);
    Register register = new Register(genName);

    int size = registerListe.size();
    int pos = -1;
    for(int i=0; i<size; i++)
      if(register.name.compareTo(registerListe.get(i).toString()) >= 0)
        pos = i;
    if(pos == size-1)
      registerListe.add(register);
    else
      registerListe.add(pos+1, register);
    return register;
  }

  static public Register[] getRegister() {
    return (Register[])registerListe.toArray(new Register[registerListe.size()]);
  }

  static public String[] getRegisterNamen() {
    int size = registerListe.size();
    String[] namen = new String[size];
    for(int i=0; i<size; i++)
      namen[i] = registerListe.get(i).toString();
    return namen;
  }

  public static void alleSummieren(ZeitIntervall intervall, boolean unterkategorien) {
    for(int i=0; i<registerListe.size(); i++) {
      Register register = (Register)registerListe.get(i);
      register.summieren(intervall, unterkategorien);
    }
  }

  public static void ersetzeKategorie(Kategorie alteKategorie, Kategorie neueKategorie) {
    for(int i=0; i<registerListe.size(); i++) {
      Register register = (Register)registerListe.get(i);
      for(int j=0;j<register.buchungen.size();j++)
        ((Buchung)register.buchungen.get(j)).ersetzeKategorie(alteKategorie, neueKategorie);
      register.fireTableDataChanged(); // Neuzeichnen der Tabelle
    }
  }

  /****************************************************************************
   * Die folgenden Methoden berschreiben Methoden der Basisklasse
   * AbstractTableModel. Sie werden von JTable verwendet.
   *
   */

  public int getColumnCount() {
    return spaltenNamen.length;
  }

  public int getRowCount() {
    return buchungen.size()+1;
  }

  public String getColumnName(int col) {
    return spaltenNamen[col];
  }

  public Class getColumnClass(int col) {
    if(col < 4)
      return Buchung.class;
    else
      return Euro.class;
  }

  public Object getValueAt(int row, int col) {
    if(col == 4) {
      if(row == buchungen.size())
        return getSaldo(row-1);
      else
        return getSaldo(row);
    }
    if(row == buchungen.size())
      return new StandardBuchung();
    return (Buchung)buchungen.get(row);
  }

  public boolean isCellEditable(int row, int col) {
    if(col == 4)
      return false;
    return true;
  }

  public void setValueAt(Object value, int row, int col) {
    if(row == buchungen.size()) {
      einfuegenBuchung((StandardBuchung)value);  // neue Buchung ist einfach
    }
    else if (col == 0) { // Datum: kill + neu einfuegen
      killBuchung((Buchung)value);
      if(value.getClass() == StandardBuchung.class)
        einfuegenBuchung((StandardBuchung)value);
      else {
        einfuegenBuchung((Umbuchung)value);
      }
    }
    else if(value.getClass() == StandardBuchung.class)
      buchungMerken((StandardBuchung)value);
    if(col == 3) // neuer Betrag -> neuerSaldo
      this.fireTableDataChanged();
  }

  /*
   * Methoden des Interface 'Speicherbar'
   */

  public void laden(DataInputStream in)
    throws IOException {
    int size = in.readInt();
    buchungen.ensureCapacity(size);
    for(int i=0; i<size; i++) {
      String typ = in.readUTF();
      if(typ.equals("Umbuchung")) {
        Umbuchung umbuchung = new Umbuchung();
        umbuchung.laden(in);
        einfuegenBuchung(umbuchung);
      }
      else if(typ.equals("StandardBuchung")) {
        StandardBuchung standardBuchung = new StandardBuchung();
        standardBuchung.laden(in);
        einfuegenBuchung(standardBuchung);
      }
      else if(!typ.equals("Dummy"))
        Applikation.fehlerAusgabe.println("-E- FATAL! Unbekannter Buchungstyp: "+typ);
      // "Dummy"-Buchung wird ignoriert!
    }
  }

  public void speichern(DataOutputStream out)
    throws IOException {
    out.writeUTF(name);
    out.writeInt(buchungen.size());
    for(int i=0; i<buchungen.size(); i++) {
      Buchung buchung = (Buchung)buchungen.get(i);
      if((buchung.getClass() == StandardBuchung.class) ||
         (((Umbuchung)buchung).getZielRegister() == this))
        buchung.speichern(out);
      else
        // "Dummy"-Buchung wird geschrieben damit die Anzahl stimmt!
        out.writeUTF("Dummy");
    }
  }

  /*
   * Methoden des Interface 'KeyListener'
   */

  public void keyReleased(KeyEvent e) {
    // weitere Tasten-Auswertungen in BuchungsEditor
    if(DEBUG)
      System.out.println("Register: KeyCode="+e.getKeyCode());

    if(e.getSource().getClass() == JTable.class) {
      JTable table = (JTable)e.getSource();
      if((e.getModifiers() == 8) && (e.getKeyCode() == 'D')) { // Alt-D
        int row = table.getSelectedRow();
        if(row == buchungen.size())
          JOptionPane.showMessageDialog(null,
            "Die Buchungsvorlage kann nicht gelscht werden!",
            "Alt-D: Register",
            JOptionPane.WARNING_MESSAGE);
        else if(row == -1) {
          table.getCellEditor().cancelCellEditing();
          JOptionPane.showMessageDialog(null,
            "Keine Zeile selektiert!",
            "Alt-D: Register",
            JOptionPane.WARNING_MESSAGE);
        }
        else {
          TableCellEditor cellEditor = table.getCellEditor();
          if(cellEditor != null)
            cellEditor.cancelCellEditing();
          killBuchung((Buchung)buchungen.get(row));
        }
      }
    }
    else
      Applikation.fehlerAusgabe.println("-E- Ups! KeyEvent von "+e.getSource());
  }

  public void keyPressed(KeyEvent e){};

  public void keyTyped(KeyEvent e){};
}