package haushalt;
import haushalt.werte.*;

import java.io.*;

/**
 * @author Lars H. Hahn
 * @version 1.2
 */

public class StandardBuchung
  extends Buchung {

  private SplitBetrag splitBetrag = new SplitBetrag();

  public StandardBuchung() {
  }

  public StandardBuchung(Datum datum, String text, SplitBetrag splitBetrag) {
    setDatum(datum);
    setText(text);
    setSplitBetrag(splitBetrag);
  }

  public String getBuchungsZeile(Register register) {
    return ""+getDatum()+"\t"+getText()+"\t"+getSplitBetrag()+"\t"+getWert();
  }

  public String getBuchungsZeile(Register register, Kategorie kategorie, boolean unterkategorien) {
    Euro wert = splitBetrag.getWert(kategorie, unterkategorien);
    if(wert.equals(Euro.NULL_EURO))
      return null;
    return ""+getDatum()+"\t"+getText()+"\t"+register+"\t"+wert;
  }

  public Object clone() {
    return new StandardBuchung(getDatum(), getText(), splitBetrag);
  }

  public void setSplitBetrag(SplitBetrag splitBetrag) {
    this.splitBetrag = splitBetrag;
  }

  public SplitBetrag getSplitBetrag() {
    return splitBetrag;
  }

  public void werteUebernehmen(StandardBuchung buchung) {  // aus einer gemerkten Buchung
    setText(buchung.getText());
    setSplitBetrag((SplitBetrag)buchung.getSplitBetrag().clone());
  }

  public Euro getWert(Kategorie kategorie, boolean unterkategorien) {
    return splitBetrag.getWert(kategorie, unterkategorien);
  }

  public Euro getWert() {
    return splitBetrag.getWert();
  }

  public void setWert(Euro wert) {
    splitBetrag.setWert(wert);
  }

  public boolean sucheText(String text) {
    return super.sucheText(text) || splitBetrag.sucheText(text);
  }

  public boolean ersetzeText(String alt, String neu) {
    return super.ersetzeText(alt, neu) || splitBetrag.ersetzeText(alt, neu);
  }

  public void ersetzeKategorie(Kategorie alteKategorie, Kategorie neueKategorie) {
    splitBetrag.ersetzeKategorie(alteKategorie, neueKategorie);
  }

  public void summieren(ZeitIntervall intervall) {
    if(getDatum().istImIntervall(intervall))
      splitBetrag.summieren(intervall);
  }

  public void laden(DataInputStream in)
    throws IOException {
    super.laden(in);
    splitBetrag.laden(in);
  }

  public void speichern(DataOutputStream out)
    throws IOException {
    out.writeUTF("StandardBuchung");
    super.speichern(out);
    splitBetrag.speichern(out);
  }
}