package haushalt;
import haushalt.werte.*;
import app.AbstractPrintablePane;

import java.awt.*;
import java.awt.geom.*;

/**
 * @author Lars H. Hahn
 * @version 1.1
 */

public class TortenPane
  extends AbstractPrintablePane {
  private final static boolean DEBUG = false;

  private final int[][] fsStandard = {
    {192, 192, 255}, {128, 128, 255}, { 64,  64, 255},
    {150,  64, 200}, {200,   0, 100}, {255,   0, 100},
    {200, 100,   0}, {200, 150,  50}, {200, 200,  50},
    {200, 255,  50}, {255, 255,  50}
  };

  private final int[][] fsSlow = {
    {192, 207, 127}, {240, 128,  96}, {111, 159, 160},
    {207, 159,  15}, {159, 128, 176}, { 80, 144, 111},
    {128, 207, 223}, {224, 207, 144}, {191, 159, 143},
    {175, 144,  96}, {144,  80, 127}
  };

  private final int[][] fsBunt = {
    {255, 128, 128}, {255, 255, 128}, {128, 255, 128},
    {128, 255, 255}, {128, 128, 255}, {255, 128, 255},
    {192,   0,   0}, {192, 192,   0}, {  0, 192, 192},
    {  0, 192, 192}, {  0,   0, 192}, {192,   0, 192}
  };

  private final int[][] fsGrau = {
    {120, 120, 120}, {140, 140, 140}, {160, 160, 160},
    {180, 180, 180}, {200, 200, 200}, {220, 220, 220},
    {200, 200, 200}, {180, 180, 180}, {160, 160, 160},
    {140, 140, 140}
  };

  private final int[][] fsPastell = {
    {140, 120, 140}, {140, 170, 210}, {120, 170, 150},
    {210, 140, 120}, {150, 140, 120}, {170, 210, 170},
    {150, 140, 120}, {210, 140, 120}, {120, 170, 150},
    {140, 170, 210}
  };

  public final static String[] schemaNamen = {
    "Standard", "Slow", "Bunt", "Grau", "Pastell"
  };

  private int farben[][];
  private TortenWert[] einnahmen;
  private TortenWert[] ausgaben;
  private Euro gesamtEinnahmen;
  private Euro gesamtAusgaben;

  public TortenPane(String ueberschrift, String farbschema,
                    TortenWert[] einnahmen, Euro gesamtEinnahmen,
                    TortenWert[] ausgaben, Euro gesamtAusgaben) {
    super(ueberschrift);
    this.einnahmen       = einnahmen;
    this.gesamtEinnahmen = gesamtEinnahmen;
    this.ausgaben        = ausgaben;
    this.gesamtAusgaben  = gesamtAusgaben;
    if(farbschema.equals(schemaNamen[0]))
      farben = fsStandard;
    else if(farbschema.equals(schemaNamen[1]))
      farben = fsSlow;
    else if(farbschema.equals(schemaNamen[2]))
      farben = fsBunt;
    else if(farbschema.equals(schemaNamen[3]))
      farben =  fsGrau;
    else if(farbschema.equals(schemaNamen[4]))
      farben =  fsPastell;
    setFont("SansSerif", 12);
  }

  public void setFont(String name, int punkt) {
    super.setFont(name, punkt);
    FontMetrics fontMetrics = getFontMetrics(getFont());
    int hoehe = 225 + (einnahmen.length + 3) * fontMetrics.getHeight();
    setPreferredSize(new Dimension(500, hoehe));
    if(DEBUG)
      System.out.println("font="+name+","+punkt+" fenster=[500,"+hoehe+"]");
  }

  public void zeichnen(Graphics g, int fensterX, int fensterY, int fensterBreite, int fensterHoehe) {
    Graphics2D g2 = (Graphics2D) g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    FontMetrics fontMetrics = g2.getFontMetrics();
    int textHoehe       = fontMetrics.getHeight();
    int maxTortenHoehe  = fensterHoehe - 14*textHoehe;
    int maxTortenBreite = fensterBreite * 18 / 40;
    int durchmesser     = (maxTortenHoehe<maxTortenBreite)?maxTortenHoehe:maxTortenBreite;
    int xEinnahmen      = durchmesser / 18 + fensterX;
    int xAusgaben       = durchmesser * 21 / 18 + fensterX;
    int yTorte          = 2*textHoehe + fensterY;
    int yText;
    int xUeberschrift;
    TortenWert[] werte;
    Euro gesamt;
    double winkel;
    double winkelSumme;
    int x;
    String ueberschrift;

    for(int i=0; i<2; i++) {
      if(i==0) {
        werte        = einnahmen;
        gesamt       = gesamtEinnahmen;
        x            = xEinnahmen;
        ueberschrift = "Einnahmen";
      }
      else {
        werte        = ausgaben;
        gesamt       = gesamtAusgaben;
        x            = xAusgaben;
        ueberschrift = "Ausgaben";
      }

      xUeberschrift = x + ((durchmesser - fontMetrics.stringWidth(ueberschrift)) >> 1);
      g2.drawString(ueberschrift, xUeberschrift, fensterY+textHoehe);
      yText = durchmesser + 3*textHoehe + fensterY;
      winkelSumme = 0.0D;

      for(int j=0; j<werte.length && werte[j]!=null; j++) {
        int farbSelect = j % farben.length;
        g2.setPaint(new Color(farben[farbSelect][0],
                              farben[farbSelect][1],
                              farben[farbSelect][2]));
        winkel = werte[j].prozent(gesamt) * 360.0D;
        g2.fill(new Arc2D.Double(x, yTorte,
                                 durchmesser, durchmesser,
                                 winkelSumme, winkel, Arc2D.PIE));
        winkelSumme += winkel;
        if(werte[j].ungleichNull()) {
          g2.fillRect(x, yText-fontMetrics.getAscent(), durchmesser, textHoehe);
          g2.setPaint(Color.black);
          g2.drawString(werte[j].getKategorie(), x+2, yText);
          String wert = werte[j].getWert().toString();
          g2.drawString(wert, x+durchmesser-fontMetrics.stringWidth(wert)-2, yText);
          yText += textHoehe;
        }
      }
    }
  }

}