package haushalt;

import app.*;
import haushalt.werte.*;
import java.awt.*;
import java.awt.geom.*;


/**
 * @author Lars H. Hahn
 * @version 1.1
 */

public class VermoegenPane
  extends AbstractPrintablePane {

  private Datum[] zeitpunkte;
  private Euro[] salden;
  private Euro grWert = Euro.NULL_EURO;
  private Euro klWert = Euro.NULL_EURO;

  public VermoegenPane(Datum[] zeitpunkte, Euro[] salden) {
    super("Vermgensentwicklung vom "+zeitpunkte[0]+" bis "+zeitpunkte[zeitpunkte.length-1]);
    this.zeitpunkte = zeitpunkte;
    this.salden = salden;
    setPreferredSize(new Dimension(500,500));

    for(int i=0; i<salden.length; i++) {
      if(salden[i].compareTo(grWert) > 0)
        grWert = salden[i];
      if(salden[i].compareTo(klWert) < 0)
        klWert = salden[i];
    }
  }

  public void zeichnen(Graphics g, int fensterX, int fensterY, int fensterBreite, int fensterHoehe) {
    Graphics2D g2 = (Graphics2D) g;
    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
    FontMetrics fontMetrics = g2.getFontMetrics();
    int textBreite      = fontMetrics.stringWidth((new Euro("99999,99")).toString());
    int textHoehe       = fontMetrics.getHeight();
    int intervallBreite = (fensterBreite - textBreite) / salden.length;
    int balkenBreite    = intervallBreite * 6 / 8;
    int xSaldoText      = (intervallBreite - textHoehe/2) * 4 / 8;
    int xBalken         = intervallBreite * 1 / 8;
    int yBalken         = fensterHoehe - textHoehe + fensterY;
    int maxBalkenHoehe  = fensterHoehe - 2 * textHoehe;
    Euro differenz      = grWert.sub(klWert);
    if(differenz.equals(Euro.NULL_EURO))
      return; // keine Vernderungen im Vermgen -> keine Anzeige

    int yLinie;
    // acht horizontale Linen + Betrag in Euro:
    for(int y = 0; y <= 8; y++) {
      double faktor = y / 8.0D;
      Euro euro = new Euro(differenz.toDouble()*faktor + klWert.toDouble());
      yLinie = yBalken - (int)(maxBalkenHoehe * faktor);
      g2.setPaint(Color.gray);
      g2.drawLine(textBreite + fensterX, yLinie, fensterBreite + fensterX, yLinie);
      g2.setPaint(Color.black);
      g2.drawString(euro.toString(),
                    fensterX + textBreite - fontMetrics.stringWidth(euro.toString()) - 3,
                    yLinie);
    }

    // klWert ist immer <= 0!
    int nullLinie = yBalken + (int)(maxBalkenHoehe * klWert.toDouble() / differenz.toDouble());
    g2.setPaint(Color.red);
    g2.drawLine(textBreite + fensterX, nullLinie, fensterBreite + fensterX, nullLinie);
    int x = textBreite + fensterX;
    for(int i=0; i<salden.length; i++) {
      int hoeheSaldo = yBalken - (int)(maxBalkenHoehe * salden[i].sub(klWert).toDouble() / differenz.toDouble());
      g2.setPaint(Color.green);
      if(salden[i].compareTo(Euro.NULL_EURO) >= 0)
        g2.fill3DRect(x+xBalken, hoeheSaldo, balkenBreite, nullLinie-hoeheSaldo, true);
      else
        g2.fill3DRect(x+xBalken, nullLinie, balkenBreite, hoeheSaldo-nullLinie, true);

      g2.setPaint(Color.black);
      g2.drawString(zeitpunkte[i].toString(), x+xBalken, fensterHoehe + fensterY - 5);

      AffineTransform oldAt = g2.getTransform();
//      g2.setPaint(Color.white);
      if(salden[i].compareTo(Euro.NULL_EURO) >= 0) {
        g2.rotate(Math.toRadians(90), x+xSaldoText, hoeheSaldo+5);
        g2.drawString(salden[i].toString(), x+xSaldoText, hoeheSaldo+5);
      }
      else {
        g2.rotate(Math.toRadians(90), x+xSaldoText, nullLinie+5);
        g2.drawString(salden[i].toString(), x+xSaldoText, nullLinie+5);
      }
      g2.setTransform(oldAt);

      x += intervallBreite;
    }

  }
}